var app = require('app');  // Module to control application life.
var BrowserWindow = require('browser-window');  // Module to create native browser window.
var constants = require("./constants");
var request = require("request");
var appName = constants.APP_NAME;
var globalShortcut = require("global-shortcut");
const dialog = require('dialog');
var path = require('path');
var fs = require('fs');
var childProcess = require('child_process');
var menu = require("menu");
var template = require("./shortcutTemplate.js");
// Report crashes to our server.
//require('crash-reporter').start();

// for windows auto update, start
var execFolderPath = path.resolve(process.execPath, "..");
// in Linux, execFolderPath is the topmost folder
var rootFolderPath = path.resolve(execFolderPath, "..");
// in Windows, rootFolderPath is the topmost folder; in Mac, it is Contents folder
var updateDotExe = path.join(rootFolderPath, "Update.exe");
var winOrLinuxResourcesPath = path.join(execFolderPath, "resources");
var winOrLinuxAppPath = path.join(winOrLinuxResourcesPath, "app");
var yiqixiePngPath =  path.join(winOrLinuxAppPath, "yiqixie.png");
var globarVariableJsPath = path.join(winOrLinuxAppPath, "globalVariable.js");
// globarVariableJsPath, used to difine window.YIQIXIE object
if (process.platform == "darwin") {
  var macResourcesPath = path.join(rootFolderPath, "Resources");
  var macAppPath = path.join(macResourcesPath, "app");
  globarVariableJsPath = path.join(macAppPath, "globalVariable.js");
}
console.log("globalVariablejs is " + globarVariableJsPath);
console.log("app version is " + app.getVersion());

var target = path.basename(process.execPath);
// target, used to create Windows desktop shortcuts
var executeSquirrelCommand = function (args, done) {
    var child = childProcess.spawn(updateDotExe, args, {detached: true});
    child.on('close', function (code) {
        done();
    });
};
var install = function (done) {
    executeSquirrelCommand(["--createShortcut", target], done);
};
var uninstall = function (done) {
    executeSquirrelCommand(["--removeShortcut", target], done);
};
/*  // here two functions are based on Squirrel.Windows, can be used as reference
var downloadUpdate = function (done) {
  var updateUrl = constants.WIN_UPDATE_URL;
  executeSquirrelCommand(["--download", updateUrl], done);
};
var installUpdate = function (done) {
  var updateUrl = constants.WIN_UPDATE_URL;
  executeSquirrelCommand(["--update", updateUrl], done);
};
*/
var handleSquirrelEvent = function () {
    if (process.platform != 'win32') {
        return false;
    }
    var squirrelEvent = process.argv[1];
    switch (squirrelEvent) {
        case '--squirrel-install':
            install(app.quit);
            return true;
        case '--squirrel-updated':
            install(app.quit);
            return true;
        case '--squirrel-obsolete':
            app.quit();
            return true;
        case '--squirrel-uninstall':
            uninstall(app.quit);
            return true;
        default:
            return true;
    }
};
// windows auto update, end

var existsUpdateExe = function () {
  var hasUpdateExe = fs.existsSync(updateDotExe);
  console.log("Have Update.exe? T/F: " + hasUpdateExe);
  return hasUpdateExe;
};

var winUpdateProcedure = function () {
  var winRequestUpdateUrl = constants.UPDATE_URL + "?platform=win32" + "&appName=" + appName;

  request(winRequestUpdateUrl, function (error, response, body) {
    try {
      var data = JSON.parse(body);
      var winUpdateUrl = data.redirectPath;

      var autoUpdater = require("auto-updater");
      console.log("updateUrl for windows is " + winUpdateUrl);
      autoUpdater.setFeedURL(winUpdateUrl);

      autoUpdater.on("error", function () {
        console.log("ERROR !");
      });
      autoUpdater.on("checking-for-update", function () {
        console.log("checking-for-update");
      });
      autoUpdater.on("update-available", function () {
        console.log("update-available");
      }),
      autoUpdater.on("update-not-available", function () {
        console.log("update-not-available");
      });
      autoUpdater.on("update-downloaded", function () {
        console.log("update-downloaded");
        dialog.showMessageBox(
                {
                  type: 'info',
                  buttons: ['稍后更新', '立即更新'],
                  message: '发现新版本，是否更新?'
                },
                function (buttonIndex) {
                  console.log("buttonIndex is " + buttonIndex);
                  if (buttonIndex == 1) {
                    autoUpdater.quitAndInstall();
                  }
                }
         );
      });
      autoUpdater.checkForUpdates();

      var shortcut = globalShortcut.register('ctrl+,+.', function () {
        console.log('ctrl+,+. is pressed');

        request(winRequestUpdateUrl, function (error, response, body) {
          try {
            var data = JSON.parse(body);
            var winUpdateUrl = data.redirectPath;
            console.log("updateUrl for windows is " + winUpdateUrl);
            autoUpdater.setFeedURL(winUpdateUrl);
            autoUpdater.checkForUpdates();
          } catch (error) {
            console.log("Error: " + error.message);
          }
        });

        dialog.showMessageBox(
                {
                  type: 'info',
                  buttons: ['OK'],
                  message: '当前版本是 ' + app.getVersion()
                },
                function (buttonIndex) {
                  console.log("buttonIndex is " + buttonIndex);
                }
         );
      });
      if (!shortcut) {
        console.log('registration shortcut failed');
      }
    } catch (error) {
      console.log("Error: " + error.message);
    }
  });
};

var isInMacPrebuilt = function() {
  var electronPath = path.join(execFolderPath, "Electron");
  var hasElectron = process.execPath == electronPath;
  console.log("Have electron? T/F: " + hasElectron);
  return hasElectron;
};

var macUpdateProcedure = function () {
  try {
      var autoUpdater = require("auto-updater");
      var updateUrl = constants.UPDATE_URL + "?platform=darwin" + "&version=" + app.getVersion() + "&appName=" + appName;
      console.log("updateUrl for mac is " + updateUrl);
      autoUpdater.setFeedURL(updateUrl);

      autoUpdater.on("error", function () {
        console.log("ERROR !");
      });
      autoUpdater.on("checking-for-update", function () {
        console.log("checking-for-update")
      });
      autoUpdater.on("update-available", function () {
        console.log("update-available")
      }),
      autoUpdater.on("update-not-available", function () {
        console.log("update-not-available")
      });
      autoUpdater.on("update-downloaded", function () {
        console.log("update-downloaded");
        dialog.showMessageBox(
                {
                  type: 'info',
                  buttons: ['稍后更新', '立即更新'],
                  message: '发现新版本，是否更新?'
                },
                function (buttonIndex) {
                  console.log("buttonIndex is " + buttonIndex);
                  if (buttonIndex == 1) {
                    autoUpdater.quitAndInstall();
                  }
                }
         );
      });
      autoUpdater.checkForUpdates();

      var shortcut = globalShortcut.register('ctrl+,+.', function () {
        console.log('ctrl+,+. is pressed');
        try {
          autoUpdater.checkForUpdates();
        } catch (error) {
          console.log("Error: " + error.message);
        }

        dialog.showMessageBox(
                {
                  type: 'info',
                  buttons: ['OK'],
                  message: '当前版本是 ' + app.getVersion()
                },
                function (buttonIndex) {
                  console.log("buttonIndex is " + buttonIndex);
                }
         );
      });
      if (!shortcut) {
        console.log('registration shortcut failed');
      }
    } catch (error) {
      console.log("Error: " + error.message);
    }
};

var linuxVersionShortCut = function () {
  var shortcut = globalShortcut.register('ctrl+,+.', function () {
    console.log('ctrl+,+. is pressed');
    dialog.showMessageBox(
            {
              type: 'info',
              buttons: ['OK'],
              message: '当前版本是 ' + app.getVersion()
            },
            function (buttonIndex) {
              console.log("buttonIndex is " + buttonIndex);
            }
     );
  });
  if (!shortcut) {
    console.log('registration shortcut failed');
  }
};

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
var mainWindow = null;

// Quit when all windows are closed.
app.on('window-all-closed', function() {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  //  if (process.platform != 'darwin') {
    app.quit();
  //  }
  mainWindow.hide();
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
app.on('ready', function() {
  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 1400,
    height: 1000,
    title: "一起写OFFICE",
    icon: yiqixiePngPath,
    'web-preferences': {
      'nodeIntegration': true,
      'preload': globarVariableJsPath,
      'web-security': false,
      'allow-displaying-insecure-content': true,
      'allow-running-insecure-content': true
    }
  });

  menu.setApplicationMenu(menu.buildFromTemplate(template));
  // and load the index.html of the app.
  //mainWindow.loadUrl('file://' + __dirname + '/index.html');
  mainWindow.loadURL(constants.YQX_URL);

  // Open the DevTools.
  //mainWindow.webContents.openDevTools({detach:true});

  // Emitted when the window is closed.
  mainWindow.on('closed', function() {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });

 // start update
  if (process.platform == 'win32') {
    console.log("hello 32");
     if (existsUpdateExe()) {
       if (handleSquirrelEvent()) {
         console.log("finished add shortcuts on windows desktop");
       }
       winUpdateProcedure();
     }
  } else if (process.platform == 'darwin') {
    if (!isInMacPrebuilt()) {
      macUpdateProcedure();
    }
  } else {
    linuxVersionShortCut();
    //TODO: linux update
  }
 // end update

});

app.on('will-quit', function () {
  // Unregister a shortcut.
  //globalShortcut.unregister('ctrl+,+.');
  // Unregister all shortcuts.
  globalShortcut.unregisterAll();
});
